<?php

function canvas($width, $height, array $color = NULL)
{
	$canvas = imagecreatetruecolor(
		$width ? $width : $this->width, 
		$height ? $height : $this->height
	);
	
	imagealphablending($canvas, false);
	imagesavealpha($canvas, true);

	if(!$color)
	$color = array(0,0,0,127);

	$color = imagecolorallocatealpha(
		$canvas, $color[0], $color[1], $color[2], count($color) > 3 ? $color[3] : 0
	);

	imagecolortransparent($canvas, $color);
	imagefill($canvas, 0, 0, $color);    

	return $canvas;
}

function resize(&$source, $width, $height, $normalize = false)
{
	$x = 0;
	$curWidth = imagesx($source);
	$curHeight = imagesy($source);
	$newImage = canvas($width, $height);
	
	if(
		$normalize &&
		$curWidth >  $width ||
		$curHeight > $height
	)
	{
		$widthCopy = $width;
		
		if(imagesy($source) > $height)
		$width = $height / $curHeight * $curWidth;
		else
		$height = $width / $curWidth * $curHeight;
		
		$x = $width > $widthCopy ? ($widthCopy - $width) / 2 : 0;
	}

	imagecopyresampled($newImage, $source, $x, 0, 0, 0, $width, $height, $curWidth, $curHeight);
	$source = $newImage;
}

/**************************************************************************************************************/

// Картинка.
$image = imagecreatefromjpeg('image.jpg');

// Маска.
$mask = imagecreatefrompng('mask.png');


// Цвет границы в формате RGBA.
$borderColor = array(0, 0, 0, 0);

// Толщина границы.
$borderWidth = 1;


// Тень.
$shadowX = -2;
$shadowY = 4;
$shadowColor = array(0, 0, 0, 70);


/**************************************************************************************************************/


imagesavealpha($mask, true);

resize($image, imagesx($mask), imagesy($mask), true);

$width = imagesx($image);
$height = imagesy($image);

$shadow = canvas($width, $height);
imagesavealpha($shadow, true);

$border = canvas($width, $height);
imagesavealpha($shadow, true);


for($x = 0; $x < $width; $x++)
{
	for($y = 0; $y < $height; $y++)
	{
		$maskRGBA = array_values(imagecolorsforindex($mask, imagecolorat($mask, $x, $y)));
		$imageRGBA = array_values(imagecolorsforindex($image, imagecolorat($image, $x, $y)));
		$_color = imagecolorallocatealpha($mask, $imageRGBA[0], $imageRGBA[1], $imageRGBA[2], $maskRGBA[3]);
		
		$_borderColor = imagecolorallocatealpha($border, $borderColor[0], $borderColor[1], $borderColor[2], $maskRGBA[3]);
		
		$_shadowColor = imagecolorallocatealpha(
			$mask, $shadowColor[0], $shadowColor[1], $shadowColor[2], 
			$maskRGBA[3] < $shadowColor[3] ? $shadowColor[3] : $maskRGBA[3]
		);
		
		imagesetpixel($image, $x, $y, $_color);
		imagesetpixel($border, $x, $y, $_borderColor);
		imagesetpixel($shadow, $x, $y, $_shadowColor);
	}
}


$newImage = canvas($width + $borderWidth * 2, $height + $borderWidth * 2);
imagealphablending($newImage, true);

resize($border, $width + $borderWidth * 2, $height + $borderWidth * 2);

imagecopy($newImage, $border, 0, 0, 0, 0, $width + $borderWidth * 2, $width + $borderWidth * 2);
imagecopy($newImage, $image, $borderWidth, $borderWidth, 0, 0, $width, $height);

$image = $newImage;
$width += $borderWidth * 2;
$height += $borderWidth * 2;


resize($shadow, $width + 1, $height + 1);

$newImage = canvas($width + abs($shadowX) + 1, $height + abs($shadowY) + 1);
imagealphablending($newImage, true);

imagecopy($newImage, $shadow, $shadowX < 0 ? 0 : $shadowX, $shadowY < 0 ? 0 : $shadowY, 0, 0, $width, $height);
imagecopy($newImage, $image, $shadowX < 0 ? abs($shadowX) : 0, $shadowY < 0 ? abs($shadowY) : 0, 0, 0, $width, $height);

$image = $newImage;

header('Content-Type: image/png');
imagepng($image);















